/*
    About: Copyright & License

    Copyright 1999-2011 (c) My-Channels
    Copyright (c) 2012-2014 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.

    Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG.

*/

////////////////////////////////////////////////////////////////////
// Simple init and custom event handler logic for our workspace app:
////////////////////////////////////////////////////////////////////
var myNick = null;

var rssChannel = null;
var stockIndicesChannel = null;
var stockIndexTradesChannel = null;
var chatRoomChannel = null;

function isLoaded() {
	showUIMessage("Initializing Session");

	var session = Nirvana.createSession({
		applicationName: "TradespaceJS",
		sessionName: "TradespaceSession",
		username: "JavaScriptTradeSpaceUser" + Math.floor(Math.random() * 101),
		drivers: [
			Nirvana.Driver.WEBSOCKET,
			Nirvana.Driver.XHR_STREAMING_CORS,
			Nirvana.Driver.XDR_STREAMING,
            Nirvana.Driver.XDR_LONGPOLL,
			Nirvana.Driver.XHR_LONGPOLL_CORS,
			Nirvana.Driver.NOXD_IFRAME_STREAMING,
			Nirvana.Driver.JSONP_LONGPOLL
		]		
	});

	session.on(Nirvana.Observe.START, function (session) {
		myNick = session.getCurrentDriver() + "-" + session.getSessionID().substr(13, 4);
		updateStatus("Session initialized with ID " + session.getSessionID());
		$(window).bind("resize", function () {
			resizeChatMessageDiv();
		}).trigger("resize");
		hideUIMessage();
	});

	session.on(Nirvana.Observe.RECONNECT, function () {
		hideUIMessage();
	});

	session.on(Nirvana.Observe.DISCONNECT, function () {
		showUIMessage("Disconnected. Reconnecting...");
	});

	session.start();

	////////////////////////////////////////////////////////////////////
	// Setup and subscribe to channels
	////////////////////////////////////////////////////////////////////

	rssChannel = session.getChannel("/fxdemo/news");
	stockIndicesChannel = session.getChannel("/fxdemo/prices");
	stockIndexTradesChannel = session.getChannel("/fxdemo/trades");
	chatRoomChannel = session.getChannel("/fxdemo/chat");

	rssChannel.on(Nirvana.Observe.DATA, rssEventCB);
	stockIndicesChannel.on(Nirvana.Observe.DATA, stockIndexEventCB);
	stockIndexTradesChannel.on(Nirvana.Observe.DATA, stockIndexTradeEventCB);
	chatRoomChannel.on(Nirvana.Observe.DATA, chatEventCB);

	chatRoomChannel.on(Nirvana.Observe.SUBSCRIBE, hideUIMessage);

	chatRoomChannel.on(Nirvana.Observe.PUBLISH, function (responseCode) {
		document.getElementById("sendbutton").disabled = false;
		document.chatform.chatinput.focus();
		updateStatus("Message sent (" + responseCode + ")");
	});

	session.subscribe([rssChannel, stockIndicesChannel, stockIndexTradesChannel, chatRoomChannel]);
}

////////////////////////////////////////////////////////////////////
// Stock Index Trades Channel Event Handler:
////////////////////////////////////////////////////////////////////

function stockIndexTradeEventCB(event) {

	var dictionary = event.getDictionary();
	var trader = dictionary.get("trader");
	var buyCCY = dictionary.get("buyCCY");
	var sellCCY = dictionary.get("sellCCY");
	var buyAmount = dictionary.get("buyAmount");
	var sellAmount = dictionary.get("sellAmount");
	var rate = dictionary.get("rate");

	var when = new Date(parseInt(event.getEventAttributes().getTimeStamp(), 10));

	var time = getTime(when);
	$("#tradesTable").find('tbody').prepend('<tr><td>' + trader + '</td><td>' + time + '</td><td>' + buyCCY + '</td><td>' + sellCCY + '</td><td>' + buyAmount + '</td><td>' + sellAmount + '</td><td>' + rate + '</td></tr>');

	$("tr:even").css("background-color", "#525369");
	$("tr:odd").css("background-color", "#434158");
}

////////////////////////////////////////////////////////////////////
// RSS News Channel Event Handler
////////////////////////////////////////////////////////////////////

var useAlt = false;

function rssEventCB(event) {
	var dictionary = event.getDictionary();
	var source = dictionary.get("chat_from");
	var link = dictionary.get("chatmsg");
	var when = new Date(parseInt(event.getEventAttributes().getTimeStamp(), 10));
	var time = getDay(when) + " " + getTime(when);


	var rssList = document.getElementById("rssList");
	var li = document.createElement("li");
	if (useAlt) {
		li.className = "alt"
	} else {
		li.className = "normal";
	}
	useAlt = (!useAlt);
	li.innerHTML = link + " <nobr>(" + source + ", " + time + ")</nobr><br/>";
	var lis = rssList.getElementsByTagName("li");
	if (lis.length > 0) {
		rssList.insertBefore(li, lis[0]);
	} else {
		rssList.appendChild(li);
	}
}


////////////////////////////////////////////////////////////////////
// Stock Indices Channel Event Handler:
////////////////////////////////////////////////////////////////////

var activeIndices = new Array();
var FXTiles = {};

function stockIndexEventCB(event) {
	var dictionary = event.getDictionary();

	var indexName = dictionary.get("name");
	var indexNameClean = indexName.replace("/", "");

	if ($.inArray(indexNameClean, activeIndices) == -1) {
		FXTiles[indexNameClean] = new FXTile(indexName, indexNameClean, dictionary.get("bid"), dictionary.get("offer"));

		var orderingIndex = 0;
		for (var i = 0; i < activeIndices.length; i++) {
			if (activeIndices[i] < indexNameClean) {
				orderingIndex = i + 1;
			}
		}

		if (orderingIndex == 0) {
			$("#trades").prepend(FXTiles[indexNameClean].getHTML());
		} else if (activeIndices.length == 0 || orderingIndex >= activeIndices.length) {
			$("#trades").append(FXTiles[indexNameClean].getHTML());
		} else {
			$("#" + activeIndices[orderingIndex]).before(FXTiles[indexNameClean].getHTML());
		}


		activeIndices.push(indexNameClean);
		activeIndices.sort();
	}
	FXTiles[indexNameClean].update(dictionary.get("bid"), dictionary.get("offer"));
}


////////////////////////////////////////////////////////////////////
// Chat Channel Event Handler:
////////////////////////////////////////////////////////////////////

var lastDay = "";

function chatEventCB(event) {
	var dictionary = event.getDictionary();
	var message = dictionary.get("message");
	var from = dictionary.get("sender");
	var when = new Date(parseInt(event.getEventAttributes().getTimeStamp(), 10));
	var day = getDate(when);
	var time = getTime(when);
	var line = "<span class='chatFrom'>[" + time + "] " + from + " : </span>";
	line += "<span class='chatMessage'>" + message + "<br/></span>";

	var msgDiv = document.getElementById("chatroom");
	if (lastDay == "") msgDiv.innerHTML = "";

	if (day != lastDay) {
		lastDay = day;
		msgDiv.innerHTML += "<div class='chatDate'>Messages on " + day + ":</div>";
	}
	msgDiv.innerHTML += line;
	resizeChatMessageDiv();
	msgDiv.scrollTop = msgDiv.scrollHeight;
}

////////////////////////////////////////////////////////////////////
// Chat Channel Additional Logic:
////////////////////////////////////////////////////////////////////

function sendChatMessage(myMessage) {
	if (myMessage == "") return false;
	document.getElementById("chatinput").value = "";
	document.getElementById("sendbutton").disabled = true;
	var event = Nirvana.createEvent();
	var dictionary = event.getDictionary();
	dictionary.putString("sender", myNick);
	dictionary.putString("message", myMessage);
	chatRoomChannel.publish(event);
	return false;
}

function buy(name) {
	FXTiles[name].buy(name)
}

function sell(name) {
	FXTiles[name].sell(name);
}

function closeTile(name) {
	$("#" + name).hide();
	$("#collapseddiv").append(FXTiles[name].getCollapsed());
}


////////////////////////////////////////////////////////////////////
// Date formatting functions:
////////////////////////////////////////////////////////////////////

function prefixZero(x) {
	if ( x < 0 || x > 9 ) {
		return x;
	}
	return "0" + x;
}

function getDay(date) { 
	var days = new Array('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	return days[date.getDay()];
}

function getTime(date) { 
        return prefixZero(date.getHours()) + ":" + prefixZero(date.getMinutes());
}

function getDate(date) {
	var months = new Array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	var year = date.getYear();
	if (year.length < 4) {
		year = "" + (year - 0 + 1900);
	}
	return date.getDate() + " " + months[date.getMonth()] + " " + year;
}
